# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# pack.py : librairie pour la génération des paquets EoleNg
#
###########################################################################

from eolepack.config import EOLEPACK_FILE, COMPIL_PATH
from eolepack.build import Build
from eolepack.errors import *
from eolepack.cfgparser import Config
from eolepack import eolepackdb
from eolepack import tools
from time import time
from os import chdir, mkdir, system
from os.path import join


def pack(paquet, version, distribution, user, password, tags=None,
         branches=None, revision=None, epoch=None, changelog=None):
    """
    toutes les étapes pour la création d'un paquet
    """

    dist_ubuntu = eolepackdb.get_ubuntu_from_dist(distribution)

    # suppression des éventuelles compilations précédentes
    old_dir = join(COMPIL_PATH, "%s-%s-*" % (paquet, version))
    system("rm -rf %s" % old_dir)

    # répertoire de compilation spécifique
    compil_path = join(COMPIL_PATH, "%s-%s-%d"%(paquet, version, time()))
    mkdir(compil_path)
    chdir(compil_path)

    # récupération des infos dans la base de données
    try:
        id_paquet = eolepackdb.get_id_paquet(paquet, version, dist_ubuntu)
    except UnknownPackage:
        return False, "Paquet %s-%s inconnu" % (paquet, version)

    # FIXME : pas sûr que la base soit à jour !
    #try:
    #    last_release = eolepackdb.get_last_release(id_paquet)
    #    release = last_release+1
    #except LastReleaseNotFound:
    #    return False, "Il n'y a aucune release du paquet %s-%s\ndans la base de données"%(paquet,version)

    config = Config(EOLEPACK_FILE)
    # Warning
    project = paquet

    # option par défaut (trunk)
    rep = ''
    rev = ''
    basedir = 'trunk'

    if branches:
        basedir = 'branches'
        rep = branches
    elif tags:
        basedir = 'tags'
        rep = tags
    elif revision:
        # on prend la révision sur la branche principale
        rev = revision

    # Instanciation de l'objet Build
    try:
        build = Build(id_paquet, paquet, version, distribution, user, password,
                      basedir=basedir, rep=rep, cfg=config, revision=rev)
    except UnknownConfig:
        return False, "Pas de configuration SVN pour %s-%s" % (paquet, version)

    # détection des sales caractères au plus tôt
    try:
        print changelog
        changelog = tools.clean_log(changelog)
    except UnicodeEncodeError:
        return False, "Problème d'encodage dans le changelog"

    # export SVN
    try:
        build.svn_export()
    except SvnExportError, msg:
        return False, "Erreur lors de l'export SVN :\n%s" % msg

    # creation tar.gz
    try:
        build.make_tarfile()
    except TarFileError:
        return False, "Erreur lors de la création de l'archive"

    # récupération changelog du TRUNK
    build.prepare_changelog()

    # on rentre dans le projet
    chdir("%s-%s" % (project, version))

    try:
        # cupération du dernier numéro de release (HUM, HUM)
        release = int(file("debian/changelog").read().split()[1].split('eole')[1][0:-1])+1
    except:
        return False, "Erreur lors de la récupération\ndu dernier numéro de release !"

    # édition changelog
    long_release = "%s-eole%d" % (version, release)
    if epoch:
        long_release = "%s:%s" % (epoch, long_release)
    try:
        build.insert_debchange(distribution, long_release, changelog)
    except ChangelogNotFound:
        return False, "Fichier changelog non trouvé"
    except DebChangeError:
        return False, "Erreur lors du debchange"

    # compilation du paquet
    try:
        build.debuild()
    except BuildError:
        return False, "Erreur lors de la compilation du paquet\nVeuillez consulter le rapport ##FIXME##"

    chdir(compil_path)

    # signature du paquet si nécessaire
    build.sign_packages()

    # vérification de la strucure SVN
    build.create_svn_structure()

    # sauvegarde du changelog
    build.commit_changelog(project, long_release, distribution)

    # FIXME : loguer le nom du gars
    eolepackdb.create_release(id_paquet, release, long_release, distribution)

    build.create_tag(release)

    # envoi sur le répository
    build.put_deb_repository(repository='eole')

    return True, "compilation réussie :\n%s-%s" % (paquet, long_release)

